/*
Eric Pavey - 2008-06-21
 PixelPlot02
 
 Based on the color values of an image, convert them
 into x,y,z coordinates, and plot them in 3d space using
 P3D renderer.  The more colors match, the bigger the given
 pixel point will grow.
 Can use either RGB or HSB color space.
 */
 
//--------------------------------
// USER VARS:

// color mode to use:  RGB or HSB
String colMode = "HSB";
// Image to sample:
String imgPath = "image.jpg";

// SYSTEM VARS:
ColorPoint[] points;
PImage srcImg;
int itter = 0;
int looping =1;
int resampleResY;
int saveSequence =0;

//---------------------------------

void setup(){
  size(512, 512, P3D);
  srcImg = loadImage(imgPath);

  // load up our array of points to draw  
  points = new ColorPoint[srcImg.pixels.length];
  for(int i=0;i<srcImg.pixels.length; i++)
    points[i] = new ColorPoint(srcImg.pixels[i]);

  noStroke();
  perspective();
  loop();

  if(colMode == "RGB")
    colorMode(RGB, 100);
  else if(colMode == "HSB")
    colorMode(HSB, 100);  
}

//---------------------------------

void draw(){
  //background(img);
  background(0,0,50);

  // position to middle of screen
  translate(width/2, height/2, -width/4);

  // have mouse control the rotation of the scene:
  rotateX(map(mouseY, height, 0, 0, TWO_PI));
  rotateY(map(mouseX, width, 0, 0, TWO_PI)) ;

  // Start plotting pixels:
  for(int i=0; i<itter; i++){
    // first check to see if this point should be drawn.  It won't
    // draw if a matching color has already been plotted, but it
    // will increase the size of the pre-existing color by 1.  
    points[i].checkDraw(i);
    // try to draw our point:
    points[i].draw();
  }

  if(itter < srcImg.pixels.length)
    itter++;
  else
    println("Finished!");
}

//---------------------------------

class ColorPoint{

  // Properties:

  // starting size
  int pSize=1;
  // has this been matched to another color?
  int matched = 0;
  // is this the "master color", that will increase
  // in size?
  int masterCol = 0;

  float xPos;
  float yPos;
  float zPos;  
  color pixCol;

  // Constructor:

  ColorPoint(color pix){
    pixCol = pix;
    // the '-100' is to help center it in the fram
    if(colMode == "RGB"){
      xPos = red(pix)-100;
      yPos = green(pix)-100;
      zPos = green(pix)-100;
    }
    else if(colMode == "HSB"){
      xPos = hue(pix)-100;
      yPos = saturation(pix)-100;
      zPos = brightness(pix)-100;
    }
  }

  // Methods:

  // Tests to see if this point should be drawn or not.  How it works:
  // Compares its color against every previous color plotted.  If it finds
  // a pre-existing match, it *won't* draw itself.  But, it does increase
  // the size of the already-plotted point.

  void checkDraw(int curDrawItter){
    if(masterCol == 0 && matched == 0){
      masterCol = 1;
      for(int i=0; i<curDrawItter-1; i++){
        if(pixCol == srcImg.pixels[i]){
          matched = 1;
          points[i].increaseSize();
          masterCol =0;
          break;
        }
      }
    }
  }

  // draw (plot) the pixel on screen.  The more matches of the same color are
  // found, increase its size.

  void draw(){
    if(masterCol == 1){
      pushMatrix();
      translate(xPos, yPos, zPos);
      fill(pixCol,75);   
      float raise = 1.0 / 3.0;
      float bSize =(pow(pSize, raise));
      box(width/100 * bSize);
      popMatrix();
    }
  }

  // increase the size of the pixel:
  void increaseSize(){
    pSize++;
  }
}
